# -*- coding: utf-8 -*-
# variance.py

import pandas as pd
import numpy as np


def variance(
    df: pd.DataFrame,
    length: int = 30,
    ddof: int = 1,
    min_periods: int = None,
    column: str = "close"
) -> pd.DataFrame:
    """Variance Indicator"""
    df_copy = df.copy()
    
    # Ensure the DataFrame contains the required column
    if column not in df.columns:
        raise KeyError(f"DataFrame must contain '{column}' column")
    
    # Validate parameters
    if length <= 1:
        raise ValueError("Length must be greater than 1")
    
    if not isinstance(ddof, int) or ddof < 0 or ddof >= length:
        raise ValueError(f"ddof must be an integer between 0 and {length-1}")
    
    # Set min_periods to length if not specified
    if min_periods is None:
        min_periods = length
    
    # Calculate variance
    df_copy[f"var_{length}"] = df_copy[column].rolling(window=length, min_periods=min_periods).var(ddof=ddof)
    
    return df_copy[[f"var_{length}"]]


variance.__doc__ = """
Name:
    Rolling Variance

Description:
    Calculates the rolling variance of a price series over a specified window length.
    Variance measures how far a set of numbers is spread out from their average value.
    It is the square of the standard deviation.

More info:
    https://en.wikipedia.org/wiki/Variance

Parameters:
    - df (pandas.DataFrame): Input DataFrame which should contain the column specified.
    - length (int): The window size for the rolling calculation. Default is 30.
    - ddof (int): Delta Degrees of Freedom. The divisor used in calculations is N - ddof, 
      where N represents the number of elements. Default is 1.
    - min_periods (int, optional): Minimum number of observations required to calculate
      a value. If None, defaults to the length parameter value.
    - column (str): The column to calculate variance on. Default is 'close'.

Call with:
    df['var_30'] = bta.variance(df)['var_30']

Returns:
    pd.DataFrame: DataFrame with variance values in a column named 'var_{length}'.
"""


def test():
    """
    Test function for the variance indicator.
    
    This function uses the generic test_indicator function from bamboo_ta.py
    to test the variance indicator.
    
    Returns:
        None: Displays the results to the console
    """
    try:
        # Import the test_indicator function from bamboo_ta
        from bamboo_ta.bamboo_ta import test_indicator
        
        # Test the indicator
        test_indicator(variance)
        
    except ImportError:
        print("Error: Could not import test_indicator from bamboo_ta.bamboo_ta")
    except Exception as e:
        print(f"Error during testing: {e}")

# Execute the test if this file is run directly
if __name__ == "__main__":
    test() 